@@ -1,7 +1,6 @@
 #!/usr/bin/python3 -B
 import argparse
 import json
-import logging
 import os
 import shutil
 import subprocess
@@ -22,7 +21,6 @@ from typing import (
 )
 
 from debputy import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR
-from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.analysis import REFERENCE_DATA_TABLE
 from debputy.analysis.debian_dir import scan_debian_dir
 from debputy.build_support import perform_clean, perform_builds
@@ -101,7 +99,6 @@ from debputy.util import (
     integrated_with_debhelper,
     PRINT_BUILD_SYSTEM_COMMAND,
     PRINT_COMMAND,
-    change_log_level,
 )
 
 
@@ -263,16 +260,11 @@ def _add_packages_args(parser: argparse.ArgumentParser) -> None:
     )
 
 
-def _build_subcommand_log_level(context: CommandContext) -> int:
-    parsed_args = context.parsed_args
-    log_level: Optional[int] = None
+def _build_subcommand_log_level(_context: CommandContext) -> int:
+    log_level: int = PRINT_BUILD_SYSTEM_COMMAND
     if os.environ.get("DH_VERBOSE", "") != "":
         log_level = PRINT_COMMAND
-    if parsed_args.debug_mode:
-        log_level = logging.INFO
-    if log_level is not None:
-        change_log_level(log_level)
-    return PRINT_BUILD_SYSTEM_COMMAND
+    return log_level
 
 
 internal_commands = ROOT_COMMAND.add_dispatching_subcommand(
@@ -357,6 +349,9 @@ def parse_args() -> argparse.Namespace:
     if hasattr(parsed_args, "packages"):
         setattr(parsed_args, "packages", frozenset(parsed_args.packages))
 
+    if os.environ.get("DEBPUTY_DEBUG", "") == "1":
+        setattr(parsed_args, "debug_mode", True)
+
     return parsed_args
 
 
@@ -1127,7 +1122,7 @@ def main() -> None:
         )
     except Exception as e:
         _error_w_stack_trace(
-            "Unhandled exception (Re-run with --debug to see the raw stack trace)",
+            "Unhandled exception (Re-run with --debug/DEBPUTY_DEBUG=1 to see the raw stack trace)",
             str(e),
             e,
             parsed_args.debug_mode,
