@@ -2,6 +2,7 @@ import argparse
 import operator
 import os
 import sys
+import textwrap
 from itertools import chain
 from typing import (
     Sequence,
@@ -28,7 +29,7 @@ from debputy.commands.debputy_cmd.output import (
 )
 from debputy.exceptions import DebputySubstitutionError
 from debputy.filesystem_scan import build_virtual_fs
-from debputy.manifest_parser.base_types import TypeMapping
+from debputy.manifest_parser.tagging_types import TypeMapping
 from debputy.manifest_parser.declarative_parser import (
     BASIC_SIMPLE_TYPES,
 )
@@ -45,13 +46,15 @@ from debputy.plugin.api.impl_types import (
     PackagerProvidedFileClassSpec,
     PluginProvidedManifestVariable,
     DispatchingParserBase,
-    SUPPORTED_DISPATCHABLE_TABLE_PARSERS,
-    OPARSER_MANIFEST_ROOT,
     PluginProvidedDiscardRule,
     AutomaticDiscardRuleExample,
     MetadataOrMaintscriptDetector,
     PluginProvidedTypeMapping,
 )
+from debputy.plugin.api.parser_tables import (
+    SUPPORTED_DISPATCHABLE_TABLE_PARSERS,
+    OPARSER_MANIFEST_ROOT,
+)
 from debputy.plugin.api.spec import (
     TypeMappingExample,
 )
@@ -121,7 +124,7 @@ TEXT_CSV_FORMAT_NO_STABILITY_PROMISE = format_output_arg(
 
 @plugin_list_cmds.register_subcommand(
     "plugins",
-    help_description="List known plugins with their versions",
+    help_description="List known plugins",
     argparser=TEXT_CSV_FORMAT_NO_STABILITY_PROMISE,
 )
 def _plugin_cmd_list_plugins(context: CommandContext) -> None:
@@ -538,7 +541,16 @@ def _plugin_cmd_show_manifest_variables(context: CommandContext) -> None:
             variable_value=None,
             is_context_specific_variable=False,
             is_documentation_placeholder=True,
-            variable_reference_documentation=f'Environment variable "{env_var}"',
+            variable_reference_documentation=textwrap.dedent(
+                f"""\
+            Environment variable "{env_var}"
+
+            Note that uses beneath `builds:` may use the environment variable defined by
+            `build-environment:` (depends on whether the rule uses eager or lazy
+            substitution) while uses outside `builds:` will generally not use a definition
+            from `build-environment:`.
+            """
+            ),
         )
     else:
         variable = variables.get(variable_name)
@@ -976,7 +988,7 @@ def _render_discard_rule(
 
 @plugin_show_cmds.register_subcommand(
     ["automatic-discard-rules", "a-d-r"],
-    help_description="Pluggable manifest rules (such as install rules)",
+    help_description="Automatic discard rules",
     argparser=add_arg(
         "discard_rule",
         metavar="automatic-discard-rule",
@@ -1033,7 +1045,7 @@ def _plugin_cmd_list_type_mappings(context: CommandContext) -> None:
 
 @plugin_show_cmds.register_subcommand(
     "type-mappings",
-    help_description="Register type mappings/descriptions",
+    help_description="Registered type mappings/descriptions",
     argparser=add_arg(
         "type_mapping",
         metavar="type-mapping",
@@ -1114,7 +1126,7 @@ def _render_type_example(
             fo.colored("Broken example: ", fg="red")
             + f"Provided example input ({v})"
             + " caused an exception when parsed. Please file a bug against the plugin."
-            + " Use --debug to see the stack trace"
+            + " Use --debug/DEBPUTY_DEBUG=1 to see the stack trace"
         )
         return fo.colored(v, fg="red") + " [Example value could not be parsed]", True
     return fo.colored(v, fg="green"), False
@@ -1170,7 +1182,7 @@ def _render_type(
             )
         )
         fo.print()
-        fo.print("Use --debug to see the stacktrace")
+        fo.print("Use --debug/DEBPUTY_DEBUG=1 to see the stacktrace")
 
 
 def _render_value(v: Any) -> str:
