@@ -17,6 +17,7 @@ from debputy.linting.lint_util import (
     TermLintReport,
     LintDiagnosticResultState,
 )
+from debputy.lsp.diagnostics import DiagnosticData
 from debputy.lsp.lsp_debian_changelog import _lint_debian_changelog
 from debputy.lsp.lsp_debian_control import (
     _lint_debian_control,
@@ -289,8 +290,9 @@ def perform_reformat(
                    - If you use `pre-commit`, then there is a formatting hook at
                      https://salsa.debian.org/debian/debputy-pre-commit-hooks
 
-                 * If you use the Debian Salsa CI pipeline, then you can set SALSA_CI_DISABLE_WRAP_AND_SORT
-                   to a "no" or 0 and `debputy` will pick up the configuration from there.
+                 * If you use the Debian Salsa CI pipeline, then you can set SALSA_CI_DISABLE_WRAP_AND_SORT (`no`)
+                   or SALSA_CI_ENABLE_WRAP_AND_SORT (`yes`) to the relevant value and `debputy` will pick up the
+                   configuration from there.
                    - Note: The option must be in `.gitlab-ci.yml` or `debian/salsa-ci.yml` to work. The Salsa CI
                      pipeline will use `wrap-and-sort` while `debputy` uses its own emulation of `wrap-and-sort`
                      (`debputy` also needs to apply the style via `debputy lsp server`).
@@ -480,6 +482,11 @@ _INITIAL_FIX_RANGE = Range(
 )
 
 
+def _is_non_interactive_auto_fix_allowed(diagnostic: Diagnostic) -> bool:
+    diag_data: Optional[DiagnosticData] = diagnostic.data
+    return diag_data is None or diag_data.get("enable_non_interactive_auto_fix", True)
+
+
 def _auto_fix_run(
     lint_context: LintContext,
     filename: str,
@@ -508,6 +515,9 @@ def _auto_fix_run(
         edits = []
         fixed_diagnostics = []
         for diagnostic in current_issues:
+            if not _is_non_interactive_auto_fix_allowed(diagnostic):
+                unfixed_diagnostics.append(diagnostic)
+                continue
             actions = provide_standard_quickfixes_from_diagnostics(
                 CodeActionParams(
                     TextDocumentIdentifier(filename),
@@ -664,15 +674,33 @@ def _diagnostics_run(
                 ),
             )
             auto_fixer = resolve_auto_fixer(filename, actions)
-            has_auto_fixer = bool(auto_fixer)
+            has_auto_fixer = bool(auto_fixer) and _is_non_interactive_auto_fix_allowed(
+                diagnostic
+            )
 
             result_state = LintDiagnosticResultState.REPORTED
             if has_auto_fixer:
-                result_state = LintDiagnosticResultState.FIXABLE
+                result_state = LintDiagnosticResultState.AUTO_FIXABLE
+            elif has_at_least_lsp_quickfix(actions):
+                result_state = LintDiagnosticResultState.MANUAL_FIXABLE
 
             lint_report.report_diagnostic(diagnostic, result_state=result_state)
 
 
+def has_at_least_lsp_quickfix(
+    actions: Optional[List[Union[Command, CodeAction]]],
+) -> bool:
+    if actions is None:
+        return False
+    for action in actions:
+        if not isinstance(action, CodeAction):
+            continue
+        if action.edit is None and action.command is None:
+            continue
+        return True
+    return False
+
+
 def resolve_auto_fixer(
     document_ref: str,
     actions: Optional[List[Union[Command, CodeAction]]],
