@@ -43,7 +43,6 @@ from debputy.util import _info, _warn
 from debputy.yaml import MANIFEST_YAML, YAMLError
 from debputy.yaml.compat import CommentedMap
 
-
 if TYPE_CHECKING:
     import lsprotocol.types as types
     from pygls.server import LanguageServer
@@ -395,7 +394,9 @@ class DebputyLanguageServer(LanguageServer):
         self.maint_preferences = MaintainerPreferenceTable({}, {})
         self.apt_cache = AptCache()
         self.background_tasks = set()
-        self.client_locale: Optional[List[str]] = None
+        self.client_locale: Optional[str] = None
+        self.forced_locale: Optional[str] = None
+        self._active_locale: Optional[List[str]] = None
 
     def finish_startup_initialization(self) -> None:
         if self._finished_initialization:
@@ -413,8 +414,24 @@ class DebputyLanguageServer(LanguageServer):
         task = self.loop.create_task(self._load_apt_cache(), name="Index apt cache")
         self.background_tasks.add(task)
         task.add_done_callback(self.background_tasks.discard)
-        self.client_locale = [params.locale] if params.locale is not None else None
-        _info(f"Client locale: {self.client_locale}")
+        self.client_locale = params.locale
+        if self.forced_locale is not None:
+            _info(
+                f"Ignoring client locale: {self.client_locale}. Using {self.forced_locale} instead as requested [--force-locale]"
+            )
+        else:
+            _info(
+                f"Client locale: {self.client_locale}. Use --force-locale to override"
+            )
+        self._update_locale()
+
+    def _update_locale(self) -> None:
+        if self.forced_locale is not None:
+            self._active_locale = [self.forced_locale]
+        elif self.client_locale is not None:
+            self._active_locale = [self.client_locale]
+        else:
+            self._active_locale = None
 
     def shutdown(self) -> None:
         for task in self.background_tasks:
@@ -425,7 +442,7 @@ class DebputyLanguageServer(LanguageServer):
     def translation(self, domain: str) -> Translations:
         return l10n.translation(
             domain,
-            languages=self.client_locale,
+            languages=self._active_locale,
         )
 
     async def _load_apt_cache(self) -> None:
