@@ -1,7 +1,12 @@
 import dataclasses
+import importlib.resources
 import os.path
 import re
 import textwrap
+
+import debputy.lsp.data.deb822_data as deb822_ref_data_dir
+
+from functools import lru_cache
 from itertools import chain
 from typing import (
     Union,
@@ -10,9 +15,8 @@ from typing import (
     Optional,
     Mapping,
     List,
-    Dict,
     Iterable,
-    TYPE_CHECKING,
+    Self,
 )
 
 from debputy.analysis.analysis_util import flatten_ppfs
@@ -33,6 +37,7 @@ from debputy.lsp.lsp_debian_control_reference_data import (
     extract_first_value_and_position,
     all_source_relationship_fields,
     StanzaMetadata,
+    SUBSTVAR_RE,
 )
 from debputy.lsp.lsp_features import (
     lint_diagnostics,
@@ -55,13 +60,17 @@ from debputy.lsp.lsp_generic_deb822 import (
     deb822_token_iter,
     deb822_format_file,
 )
+from debputy.lsp.lsp_reference_keyword import LSP_DATA_DOMAIN
 from debputy.lsp.quickfixes import (
-    propose_remove_line_quick_fix,
-    range_compatible_with_remove_line_fix,
     propose_correct_text_quick_fix,
     propose_insert_text_on_line_after_diagnostic_quick_fix,
     propose_remove_range_quick_fix,
 )
+from debputy.lsp.ref_models.deb822_reference_parse_models import (
+    DCTRL_SUBSTVARS_REFERENCE_DATA_PARSER,
+    DCtrlSubstvar,
+)
+from debputy.lsp.text_util import markdown_urlify
 from debputy.lsp.vendoring._deb822_repro import (
     Deb822FileElement,
     Deb822ParagraphElement,
@@ -70,14 +79,12 @@ from debputy.lsp.vendoring._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
 )
 from debputy.lsprotocol.types import (
-    Range,
     Position,
     FoldingRange,
     FoldingRangeParams,
     CompletionItem,
     CompletionList,
     CompletionParams,
-    DiagnosticRelatedInformation,
     Location,
     HoverParams,
     Hover,
@@ -91,17 +98,14 @@ from debputy.lsprotocol.types import (
     InlayHint,
     InlayHintLabelPart,
 )
+from debputy.manifest_parser.util import AttributePath
 from debputy.packager_provided_files import (
     PackagerProvidedFile,
     detect_all_packager_provided_files,
 )
 from debputy.plugin.api.impl import plugin_metadata_for_debputys_own_plugin
-from debputy.util import detect_possible_typo, PKGNAME_REGEX, _info
-
-if TYPE_CHECKING:
-    pass
-else:
-    pass
+from debputy.util import PKGNAME_REGEX, _info
+from debputy.yaml import MANIFEST_YAML
 
 try:
     from debputy.lsp.vendoring._deb822_repro.locatable import (
@@ -132,7 +136,8 @@ class SubstvarMetadata:
     name: str
     defined_by: str
     dh_sequence: Optional[str]
-    source: Optional[str]
+    doc_uris: Sequence[str]
+    synopsis: str
     description: str
 
     def render_metadata_fields(self) -> str:
@@ -140,8 +145,27 @@ class SubstvarMetadata:
         dh_seq = (
             f"DH Sequence: {self.dh_sequence}" if self.dh_sequence is not None else None
         )
-        source = f"Source: {self.source}" if self.source is not None else None
-        return "\n".join(filter(None, (def_by, dh_seq, source)))
+        doc_uris = self.doc_uris
+        parts = [def_by, dh_seq]
+        if doc_uris:
+            if len(doc_uris) == 1:
+                parts.append(f"Documentation: {markdown_urlify(doc_uris[0])}")
+            else:
+                parts.append("Documentation:")
+                parts.extend(f" - {markdown_urlify(uri)}" for uri in doc_uris)
+        return "\n".join(parts)
+
+    @classmethod
+    def from_ref_data(cls, x: DCtrlSubstvar) -> "Self":
+        doc = x.get("documentation", {})
+        return cls(
+            x["name"],
+            x["defined_by"],
+            x.get("dh_sequence"),
+            doc.get("uris", []),
+            doc.get("synopsis", ""),
+            doc.get("long_description", ""),
+        )
 
 
 def relationship_substvar_for_field(substvar: str) -> Optional[str]:
@@ -153,154 +177,33 @@ def relationship_substvar_for_field(substvar: str) -> Optional[str]:
     return relationship_fields.get(substvar[col_idx + 1 : -1].lower())
 
 
-def _substvars_metadata(*args: SubstvarMetadata) -> Mapping[str, SubstvarMetadata]:
+def _as_substvars_metadata(
+    args: List[SubstvarMetadata],
+) -> Mapping[str, SubstvarMetadata]:
     r = {s.name: s for s in args}
     assert len(r) == len(args)
     return r
 
 
-_SUBSTVAR_RE = re.compile(r"[$][{][a-zA-Z0-9][a-zA-Z0-9-:]*[}]")
-_SUBSTVARS_DOC = _substvars_metadata(
-    SubstvarMetadata(
-        "${}",
-        "`dpkg-gencontrol`",
-        "(default)",
-        "<https://manpages.debian.org/deb-substvars.5>",
-        textwrap.dedent(
-            """\
-            This is a substvar for a literal `$`. This form will never recurse
-            into another substvar. As an example, `${}{binary:Version}` will result
-            literal `${binary:Version}` (which will not be replaced).
-        """
-        ),
-    ),
-    SubstvarMetadata(
-        "${binary:Version}",
-        "`dpkg-gencontrol`",
-        "(default)",
-        "<https://manpages.debian.org/deb-substvars.5>",
-        textwrap.dedent(
-            """\
-            The version of the current binary package including binNMU version.
+def substvars_metadata_basename() -> str:
+    return "debian_control_substvars_data.yaml"
 
-            Often used with `Depends: dep (= ${binary:Version})` relations
-            where:
 
-             * The `dep` package is from the same source (listed in the same
-               `debian/control` file)
-             * The current package and `dep` are both `arch:any` (or both `arch:all`)
-               packages.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${source:Version}",
-        "`dpkg-gencontrol`",
-        "(default)",
-        "<https://manpages.debian.org/deb-substvars.5>",
-        textwrap.dedent(
-            """\
-            The version of the current source package excluding binNMU version.
+@lru_cache
+def substvars_metadata() -> Mapping[str, SubstvarMetadata]:
+    p = importlib.resources.files(deb822_ref_data_dir.__name__).joinpath(
+        substvars_metadata_basename()
+    )
 
-            Often used with `Depends: dep (= ${source:Version})` relations
-            where:
+    with p.open("r", encoding="utf-8") as fd:
+        raw = MANIFEST_YAML.load(fd)
 
-             * The `dep` package is from the same source (listed in the same
-               `debian/control` file)
-             * The `dep` is `arch:all`.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${misc:Depends}",
-        "`debhelper`",
-        "(default)",
-        "<https://manpages.debian.org/debhelper.7>",
-        textwrap.dedent(
-            """\
-            Some debhelper commands may make the generated package need to depend on some other packages.
-            For example, if you use `dh_installdebconf(1)`, your package will generally need to depend on
-            debconf. Or if you use `dh_installxfonts(1)`, your package will generally need to depend on a
-            particular version of xutils. Keeping track of these miscellaneous dependencies can be
-            annoying since they are dependent on how debhelper does things, so debhelper offers a way to
-            automate it.
+    attr_path = AttributePath.root_path(p)
+    ref = DCTRL_SUBSTVARS_REFERENCE_DATA_PARSER.parse_input(raw, attr_path)
+    return _as_substvars_metadata(
+        [SubstvarMetadata.from_ref_data(x) for x in ref["substvars"]]
+    )
 
-            All commands of this type, besides documenting what dependencies may be needed on their man
-            pages, will automatically generate a substvar called ${misc:Depends}. If you put that token
-            into your `debian/control` file, it will be expanded to the dependencies debhelper figures
-            you need.
-
-            This is entirely independent of the standard `${shlibs:Depends}` generated by `dh_makeshlibs(1)`,
-            and the `${perl:Depends}` generated by `dh_perl(1)`.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${misc:Pre-Depends}",
-        "`debhelper`",
-        "(default)",
-        None,
-        textwrap.dedent(
-            """\
-            This is the moral equivalent to `${misc:Depends}` but for `Pre-Depends`.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${perl:Depends}",
-        "`dh_perl`",
-        "(default)",
-        "<https://manpages.debian.org/dh_perl.1>",
-        textwrap.dedent(
-            """\
-            The dependency on perl as determined by `dh_perl`. Note this only covers the relationship
-            with the Perl interpreter and not perl modules.
-
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${gir:Depends}",
-        "`dh_girepository`",
-        "gir",
-        "<https://manpages.debian.org/dh_girepository.1>",
-        textwrap.dedent(
-            """\
-            Dependencies related to GObject introspection data.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${shlibs:Depends}",
-        "`dpkg-shlibdeps` (often via `dh_shlibdeps`)",
-        "(default)",
-        "<https://manpages.debian.org/dpkg-shlibdeps.1>",
-        textwrap.dedent(
-            """\
-            Dependencies related to ELF dependencies.
-    """
-        ),
-    ),
-    SubstvarMetadata(
-        "${shlibs:Pre-Depends}",
-        "`dpkg-shlibdeps` (often via `dh_shlibdeps`)",
-        "(default)",
-        "<https://manpages.debian.org/dpkg-shlibdeps.1>",
-        textwrap.dedent(
-            """\
-            Dependencies related to ELF dependencies. The `Pre-Depends`
-            version is often only seen in `Essential: yes` packages
-            or packages that manually request the `Pre-Depends`
-            relation via `dpkg-shlibdeps`.
-
-            Note: This substvar only appears in `debhelper-compat (= 14)`, or
-            with use of `debputy` (at an integration level, where `debputy`
-            runs `dpkg-shlibdeps`), or when passing relevant options to
-            `dpkg-shlibdeps` (often via `dh_shlibdeps`) such as `-dPre-Depends`.
-    """
-        ),
-    ),
-)
 
 _DCTRL_FILE_METADATA = DctrlFileMetadata()
 
@@ -640,8 +543,8 @@ def _custom_hover(
     except (ValueError, IndexError):
         pass
 
-    if substvar == "${}" or _SUBSTVAR_RE.fullmatch(substvar):
-        substvar_md = _SUBSTVARS_DOC.get(substvar)
+    if substvar == "${}" or SUBSTVAR_RE.fullmatch(substvar):
+        substvar_md = substvars_metadata().get(substvar)
 
         computed_doc = ""
         for_field = relationship_substvar_for_field(substvar)
@@ -660,7 +563,10 @@ def _custom_hover(
             doc = f"No documentation for {substvar}.\n"
             md_fields = ""
         else:
-            doc = substvar_md.description
+            doc = ls.translation(LSP_DATA_DOMAIN).pgettext(
+                f"Substvars:{substvar_md.name}",
+                substvar_md.description,
+            )
             md_fields = "\n" + substvar_md.render_metadata_fields()
         return f"# Substvar `{substvar}`\n\n{doc}{computed_doc}{md_fields}"
 
@@ -721,7 +627,10 @@ def _doc_inlay_hint(
         )
         stanza_def = _DCTRL_FILE_METADATA.classify_stanza(stanza, stanza_no)
         for known_field in stanza_def.stanza_fields.values():
-            if not known_field.inherits_from_source or known_field.name in stanza:
+            if (
+                not known_field.inheritable_from_other_stanza
+                or known_field.name in stanza
+            ):
                 continue
 
             inherited_value = source_stanza.get(known_field.name)
@@ -857,170 +766,6 @@ def _binary_package_checks(
         )
 
 
-def _diagnostics_for_stanza(
-    deb822_file: Deb822FileElement,
-    stanza: Deb822ParagraphElement,
-    stanza_position: "TEPosition",
-    source_stanza: Deb822ParagraphElement,
-    stanza_metadata: StanzaMetadata[DctrlKnownField],
-    other_stanza_metadata: StanzaMetadata[DctrlKnownField],
-    is_binary_paragraph: bool,
-    doc_reference: str,
-    lint_state: LintState,
-) -> None:
-    _, representation_field_range = stanza_metadata.stanza_representation(
-        stanza,
-        stanza_position,
-    )
-    known_fields = stanza_metadata.stanza_fields
-    other_known_fields = other_stanza_metadata.stanza_fields
-    stanza_metadata.omitted_field_diagnostics(
-        lint_state,
-        deb822_file,
-        stanza,
-        stanza_position,
-        inherit_from_stanza=source_stanza,
-        representation_field_range=representation_field_range,
-    )
-
-    if is_binary_paragraph:
-        _binary_package_checks(
-            stanza,
-            stanza_position,
-            source_stanza,
-            representation_field_range,
-            lint_state,
-        )
-    else:
-        _source_package_checks(
-            stanza,
-            stanza_position,
-            stanza_metadata,
-            lint_state,
-        )
-
-    seen_fields: Dict[str, Tuple[str, str, Range, List["TERange"]]] = {}
-
-    for kvpair in stanza.iter_parts_of_type(Deb822KeyValuePairElement):
-        field_name_token = kvpair.field_token
-        field_name = field_name_token.text
-        field_name_lc = field_name.lower()
-        normalized_field_name_lc = stanza_metadata.normalize_field_name(field_name_lc)
-        known_field = known_fields.get(normalized_field_name_lc)
-        field_value = stanza[field_name]
-        kvpair_range = kvpair.range_in_parent().relative_to(stanza_position)
-        field_range = kvpair.field_token.range_in_parent().relative_to(
-            kvpair_range.start_pos
-        )
-        field_position_te = field_range.start_pos
-        field_name_typo_detected = False
-        existing_field_range = seen_fields.get(normalized_field_name_lc)
-        if existing_field_range is not None:
-            existing_field_range[3].append(field_range)
-        else:
-            normalized_field_name = stanza_metadata.normalize_field_name(field_name)
-            seen_fields[field_name_lc] = (
-                field_name,
-                normalized_field_name,
-                field_range,
-                [],
-            )
-
-        if known_field is None:
-            candidates = detect_possible_typo(normalized_field_name_lc, known_fields)
-            if candidates:
-                known_field = known_fields[candidates[0]]
-                token_range = TERange.from_position_and_size(
-                    field_position_te,
-                    kvpair.field_token.size(),
-                )
-                field_name_typo_detected = True
-                lint_state.emit_diagnostic(
-                    token_range,
-                    f'The "{field_name}" looks like a typo of "{known_field.name}".',
-                    "warning",
-                    "debputy",
-                    quickfixes=[
-                        propose_correct_text_quick_fix(known_fields[m].name)
-                        for m in candidates
-                    ],
-                )
-        if known_field is None:
-            known_else_where = other_known_fields.get(normalized_field_name_lc)
-            if known_else_where is not None:
-                intended_usage = "Source" if is_binary_paragraph else "Package"
-                lint_state.emit_diagnostic(
-                    field_name,
-                    f'The {field_name} is defined for use in the "{intended_usage}" stanza.'
-                    f" Please move it to the right place or remove it",
-                    "error",
-                    known_else_where.missing_field_authority,
-                )
-            continue
-
-        if field_value.strip() == "":
-            lint_state.emit_diagnostic(
-                field_range,
-                f"The {field_name} has no value. Either provide a value or remove it.",
-                "error",
-                "Policy 5.1",
-            )
-            continue
-        known_field.field_diagnostics(
-            deb822_file,
-            kvpair,
-            stanza,
-            stanza_position,
-            kvpair_range,
-            lint_state,
-            field_name_typo_reported=field_name_typo_detected,
-        )
-
-        source_value = source_stanza.get(field_name)
-
-        if known_field.inherits_from_source and field_value == source_value:
-            if range_compatible_with_remove_line_fix(field_range):
-                fix_data = propose_remove_line_quick_fix()
-            else:
-                fix_data = None
-            lint_state.emit_diagnostic(
-                field_range,
-                f"The field {field_name} duplicates the value from the Source stanza.",
-                "informational",
-                "debputy",
-                quickfixes=fix_data,
-            )
-    for (
-        field_name,
-        normalized_field_name,
-        field_range,
-        duplicates,
-    ) in seen_fields.values():
-        if not duplicates:
-            continue
-        related_information = [
-            DiagnosticRelatedInformation(
-                location=Location(doc_reference, field_range),
-                message=f"First definition of {field_name}",
-            )
-        ]
-        related_information.extend(
-            DiagnosticRelatedInformation(
-                location=Location(doc_reference, r),
-                message=f"Duplicate of {field_name}",
-            )
-            for r in duplicates
-        )
-        for dup_range in duplicates:
-            lint_state.emit_diagnostic(
-                dup_range,
-                f"The {normalized_field_name} field name was used multiple times in this stanza.",
-                "error",
-                "Policy 5.1",
-                related_information=related_information,
-            )
-
-
 def _scan_for_syntax_errors_and_token_level_diagnostics(
     deb822_file: Deb822FileElement,
     lint_state: LintState,
@@ -1086,34 +831,52 @@ def _lint_debian_control(lint_state: LintState) -> None:
         lint_state,
     )
 
-    paragraphs = list(deb822_file)
-    source_paragraph = paragraphs[0] if paragraphs else None
+    stanzas = list(deb822_file)
+    source_stanza = stanzas[0] if stanzas else None
     binary_stanzas_w_pos = []
 
-    source_stanza, binary_stanza = _DCTRL_FILE_METADATA.stanza_types()
+    source_stanza_metadata, binary_stanza_metadata = _DCTRL_FILE_METADATA.stanza_types()
 
-    for paragraph_no, paragraph in enumerate(paragraphs, start=1):
-        paragraph_pos = paragraph.position_in_file()
-        if paragraph_pos.line_position >= first_error:
+    for stanza_no, stanza in enumerate(stanzas, start=1):
+        stanza_position = stanza.position_in_file()
+        if stanza_position.line_position >= first_error:
             break
-        is_binary_paragraph = paragraph_no != 1
-        if is_binary_paragraph:
-            stanza_metadata = binary_stanza
-            other_stanza_metadata = source_stanza
-            binary_stanzas_w_pos.append((paragraph, paragraph_pos))
+        is_binary_stanza = stanza_no != 1
+        if is_binary_stanza:
+            stanza_metadata = binary_stanza_metadata
+            other_stanza_metadata = source_stanza_metadata
+            other_stanza_name = "Source"
+            binary_stanzas_w_pos.append((stanza, stanza_position))
+            _, representation_field_range = stanza_metadata.stanza_representation(
+                stanza, stanza_position
+            )
+            _binary_package_checks(
+                stanza,
+                stanza_position,
+                source_stanza,
+                representation_field_range,
+                lint_state,
+            )
         else:
-            stanza_metadata = source_stanza
-            other_stanza_metadata = binary_stanza
-        _diagnostics_for_stanza(
+            stanza_metadata = source_stanza_metadata
+            other_stanza_metadata = binary_stanza_metadata
+            other_stanza_name = "Binary"
+            _source_package_checks(
+                stanza,
+                stanza_position,
+                stanza_metadata,
+                lint_state,
+            )
+
+        stanza_metadata.stanza_diagnostics(
             deb822_file,
-            paragraph,
-            paragraph_pos,
-            source_paragraph,
-            stanza_metadata,
-            other_stanza_metadata,
-            is_binary_paragraph,
+            stanza,
+            stanza_position,
             doc_reference,
             lint_state,
+            confusable_with_stanza_metadata=other_stanza_metadata,
+            confusable_with_stanza_name=other_stanza_name,
+            inherit_from_stanza=source_stanza if is_binary_stanza else None,
         )
 
     _detect_misspelled_packaging_files(
