@@ -115,7 +115,10 @@ def _lint_debian_debputy_manifest(lint_state: LintState) -> None:
             column,
         )
         lint_state.emit_diagnostic(
-            error_range, f"YAML parse error: {e}", "error", "debputy"
+            error_range,
+            f"YAML parse error: {e}",
+            "error",
+            "debputy",
         )
     except YAMLError as e:
         error_range = TERange(
