@@ -1581,7 +1581,7 @@ def load_plugin_features(
             if debug_mode:
                 _warn(
                     f"The optional plugin {plugin_metadata.plugin_name} failed during load. Re-raising due"
-                    f" to --debug/-d."
+                    f" to --debug/-d or DEBPUTY_DEBUG=1"
                 )
                 raise
             try:
@@ -1596,7 +1596,8 @@ def load_plugin_features(
             else:
                 _warn(
                     f"The optional plugin {plugin_metadata.plugin_name} failed during load. The plugin was"
-                    f" deactivated. Use debug mode (--debug) to show the stacktrace (the warning will become an error)"
+                    f" deactivated. Use debug mode (--debug/DEBPUTY_DEBUG=1) to show the stacktrace"
+                    f" (the warning will become an error)"
                 )
 
     return plugin_feature_set
