@@ -1258,7 +1258,8 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
         if "DEB_PYTHON_INSTALL_LAYOUT" not in os.environ:
             replacements["DEB_PYTHON_INSTALL_LAYOUT"] = "deb"
         if "PKG_CONFIG" not in os.environ:
-            replacements["PKG_CONFIG"] = build_context.cross_tool("pkg-config")
+            pkg_config = build_context.cross_tool("pkg-config")
+            replacements["PKG_CONFIG"] = f"/usr/bin/{pkg_config}"
         return EnvironmentModification(
             replacements=tuple((k, v) for k, v in replacements.items())
         )
@@ -1347,6 +1348,8 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
         ]
         if not context.is_terse_build:
             cmake_flags.append("-DCMAKE_VERBOSE_MAKEFILE=ON")
+        if not context.should_run_tests:
+            cmake_flags.append("-DBUILD_TESTING:BOOL=OFF")
 
         self._compiler_and_cross_flags(context, cmake_flags)
 
@@ -1362,8 +1365,8 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
         if "CPPFLAGS" in os.environ:
             # CMake doesn't respect CPPFLAGS, see #653916.
             cppflags = os.environ["CPPFLAGS"]
-            cflags = os.environ.get("CFLAGS", "") + f" {cppflags}".lstrip()
-            cxxflags = os.environ.get("CXXFLAGS", "") + f" {cppflags}".lstrip()
+            cflags = f"{os.environ.get('CFLAGS', '')} {cppflags}".lstrip()
+            cxxflags = f"{os.environ.get('CXXFLAGS', '')} {cppflags}".lstrip()
             env_mod = env_mod.combine(
                 # The debhelper build system never showed this delta, so people might find it annoying.
                 EnvironmentModification(
@@ -1590,6 +1593,7 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
         meson_flags = [
             "--wrap-mode=nodownload",
             "--buildtype=plain",
+            "--prefix=/usr",
             "--sysconfdir=/etc",
             "--localstatedir=/var",
             f"--libdir=lib/{dpkg_architecture_variables.current_host_multiarch}",
@@ -1664,6 +1668,7 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
             "install",
             "--destdir",
             dest_dir,
+            cwd=self.build_directory,
             env_mod=self._default_meson_env(),
         )
 
