@@ -34,7 +34,6 @@ from debian.deb822 import Deb822
 
 from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
 from debputy.exceptions import DebputySubstitutionError
-from debputy.types import EnvironmentModification
 
 try:
     from Levenshtein import distance
@@ -73,6 +72,7 @@ else:
 
 
 if TYPE_CHECKING:
+    from debputy.types import EnvironmentModification
     from debputy.packages import BinaryPackage
     from debputy.substitution import Substitution
 
@@ -112,10 +112,12 @@ _STDOUT_HANDLER: Optional[logging.StreamHandler[Any]] = None
 _STDERR_HANDLER: Optional[logging.StreamHandler[Any]] = None
 PRINT_COMMAND = logging.INFO + 3
 PRINT_BUILD_SYSTEM_COMMAND = PRINT_COMMAND + 3
+TRACE_LOG = logging.DEBUG + 3
 
 # Map them back to `INFO`. The names must be unique so the prefix is stripped.
 logging.addLevelName(PRINT_COMMAND, "__INFO")
 logging.addLevelName(PRINT_BUILD_SYSTEM_COMMAND, "_INFO")
+logging.addLevelName(TRACE_LOG, "TRACE")
 
 
 def assume_not_none(x: Optional[T]) -> T:
@@ -141,6 +143,20 @@ def _info(msg: str) -> None:
     # No fallback print for info
 
 
+def _is_trace_log_enabled() -> bool:
+    global _DEFAULT_LOGGER
+    logger = _DEFAULT_LOGGER
+    return logger is not None and logger.isEnabledFor(TRACE_LOG)
+
+
+def _trace_log(msg: str) -> None:
+    global _DEFAULT_LOGGER
+    logger = _DEFAULT_LOGGER
+    if logger:
+        logger.log(TRACE_LOG, msg)
+    # No fallback print for this level
+
+
 def _is_debug_log_enabled() -> bool:
     global _DEFAULT_LOGGER
     logger = _DEFAULT_LOGGER
@@ -152,7 +168,7 @@ def _debug_log(msg: str) -> None:
     logger = _DEFAULT_LOGGER
     if logger:
         logger.debug(msg)
-    # No fallback print for info
+    # No fallback print for this level
 
 
 def _error(msg: str, *, prog: Optional[str] = None) -> "NoReturn":
@@ -266,7 +282,7 @@ def escape_shell(*args: str) -> str:
 def render_command(
     *args: str,
     cwd: Optional[str] = None,
-    env_mod: Optional[EnvironmentModification] = None,
+    env_mod: Optional["EnvironmentModification"] = None,
 ) -> str:
     env_mod_prefix = ""
     if env_mod:
@@ -289,7 +305,7 @@ def render_command(
 def print_command(
     *args: str,
     cwd: Optional[str] = None,
-    env_mod: Optional[EnvironmentModification] = None,
+    env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_COMMAND,
 ) -> None:
     if _DEFAULT_LOGGER is None or not _DEFAULT_LOGGER.isEnabledFor(print_at_log_level):
@@ -307,7 +323,7 @@ def run_command(
     *args: str,
     cwd: Optional[str] = None,
     env: Optional[Mapping[str, str]] = None,
-    env_mod: Optional[EnvironmentModification] = None,
+    env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_COMMAND,
 ) -> None:
     print_command(
@@ -335,7 +351,7 @@ def run_build_system_command(
     *args: str,
     cwd: Optional[str] = None,
     env: Optional[Mapping[str, str]] = None,
-    env_mod: Optional[EnvironmentModification] = None,
+    env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_BUILD_SYSTEM_COMMAND,
 ) -> None:
     run_command(
@@ -530,10 +546,20 @@ def scratch_dir(*, create_if_not_exists: bool = True) -> str:
     if create_if_not_exists:
         ensure_dir(_SCRATCH_DIR)
         if is_debputy_dir:
-            Path("debian/.debputy/.gitignore").write_text("*\n")
+            generated_root_directory("debian/.debputy", internal_only=True)
     return _SCRATCH_DIR
 
 
+def generated_root_directory(path: str, *, internal_only: bool = False) -> None:
+    root_dir = Path(path)
+    (root_dir / ".gitignore").write_text("*\n")
+    # TODO: Should we add a "CACHEDIR.TAG" here? (Requires a relevant ignore rule
+    if internal_only:
+        (root_dir / "CACHEDIR.TAG").write_bytes(
+            b"Signature: 8a477f597d28d172789f06886806bc55"
+        )
+
+
 _RUNTIME_CONTAINER_DIR_KEY: Optional[str] = None
 
 
